@@ -12,7 +12,7 @@
 
 namespace GlobalConstants
 {
-    const int GAMEDETAILS_CACHE_VERSION = 3;
+    const int GAMEDETAILS_CACHE_VERSION = 4;
     const int ZLIB_WINDOW_SIZE = 15;
 
     struct optionsStruct {const unsigned int id; const std::string code; const std::string str; const std::string regexp;};
@@ -46,6 +46,7 @@ namespace GlobalConstants
     const unsigned int LANGUAGE_AR = 1 << 24;
     const unsigned int LANGUAGE_RO = 1 << 25;
     const unsigned int LANGUAGE_HE = 1 << 26;
+    const unsigned int LANGUAGE_TH = 1 << 27;
 
     const std::vector<optionsStruct> LANGUAGES =
     {
@@ -75,7 +76,8 @@ namespace GlobalConstants
         { LANGUAGE_ES_419, "es_mx", "Spanish (Latin American)", "es_mx|es-mx|esmx|es-419|spanish_latin_american" },
         { LANGUAGE_AR, "ar", "Arabic",  "ar|ara|arabic|ar[_-][A-Z]{2}"    },
         { LANGUAGE_RO, "ro", "Romanian",  "ro|ron|rum|romanian|ro[_-][RM]O" },
-        { LANGUAGE_HE, "he", "Hebrew",  "he|heb|hebrew|he[_-]IL" }
+        { LANGUAGE_HE, "he", "Hebrew",  "he|heb|hebrew|he[_-]IL" },
+        { LANGUAGE_TH, "th", "Thai",  "th|tha|thai|th[_-]TH" }
     };
 
     // Platform constants
@@ -104,12 +106,16 @@ namespace GlobalConstants
     const unsigned int CDN_EDGECAST  = 1 << 0;
     const unsigned int CDN_HIGHWINDS = 1 << 1;
     const unsigned int CDN_GOG       = 1 << 2;
+    const unsigned int CDN_LUMEN     = 1 << 3;
+    const unsigned int CDN_AKAMAI    = 1 << 4;
 
     const std::vector<optionsStruct> GALAXY_CDNS =
     {
         { CDN_EDGECAST,  "edgecast",   "Edgecast",  "ec|edgecast"             },
         { CDN_HIGHWINDS, "high_winds", "Highwinds", "hw|highwinds|high_winds" },
-        { CDN_GOG,       "gog_cdn",    "GOG",       "gog|gog_cdn"             }
+        { CDN_GOG,       "gog_cdn",    "GOG",       "gog|gog_cdn"             },
+        { CDN_LUMEN,     "lumen",      "Lumen",     "lumen|lumen_cdn"         },
+        { CDN_AKAMAI,    "akamai_edgecast_proxy", "Akamai", "akamai|akamai_cdn|akamai_ec|akamai_edgecast_proxy" }
     };
 }
 
