@@ -8,13 +8,13 @@
 #include "config.h"
 #include "util.h"
 #include "globalconstants.h"
-#include "ssl_thread_setup.h"
 #include "galaxyapi.h"
 #include "globals.h"
 
 #include <fstream>
 #include <boost/filesystem.hpp>
 #include <boost/program_options.hpp>
+#include <signal.h>
 
 namespace bpo = boost::program_options;
 Config Globals::globalConfig;
@@ -26,6 +26,13 @@ template<typename T> void set_vm_value(std::map<std::string, bpo::variable_value
 
 int main(int argc, char *argv[])
 {
+    struct sigaction act;
+    act.sa_handler = SIG_IGN;
+    act.sa_flags = SA_RESTART;
+    sigemptyset(&act.sa_mask);
+    if (sigaction(SIGPIPE, &act, NULL) < 0)
+        return 1;
+
     rhash_library_init();
 
     // Constants for option selection with include/exclude
@@ -132,7 +139,7 @@ int main(int argc, char *argv[])
     std::string check_orphans_text = "Check for orphaned files (files found on local filesystem that are not found on GOG servers). Sets regular expression filter (Perl syntax) for files to check. If no argument is given then the regex defaults to '" + orphans_regex_default + "'";
 
     // Help text for subdir options
-    std::string subdir_help_text = "\nTemplates:\n- %platform%\n- %gamename%\n- %dlcname%";
+    std::string subdir_help_text = "\nTemplates:\n- %platform%\n- %gamename%\n- %gamename_firstletter%\n- %dlcname%";
 
     // Help text for include and exclude options
     std::string include_options_text;
@@ -144,6 +151,7 @@ int main(int argc, char *argv[])
 
     std::string galaxy_product_id_install;
     std::string galaxy_product_id_show_builds;
+    std::string tags;
 
     std::vector<std::string> vFileIdStrings;
     std::vector<std::string> unrecognized_options_cfg;
@@ -168,6 +176,7 @@ int main(int argc, char *argv[])
         bool bNoSubDirectories = false;
         bool bNoPlatformDetection = false;
         bool bNoGalaxyDependencies = false;
+        bool bUseDLCList = false;
         std::string sInstallerPlatform;
         std::string sInstallerLanguage;
         std::string sIncludeOptions;
@@ -207,6 +216,8 @@ int main(int argc, char *argv[])
 #ifdef USE_QT_GUI_LOGIN
             ("enable-login-gui", bpo::value<bool>(&Globals::globalConfig.bEnableLoginGUI)->zero_tokens()->default_value(false), "Enable login GUI when encountering reCAPTCHA on login form")
 #endif
+            ("list-tags", bpo::value<bool>(&Globals::globalConfig.bListTags)->zero_tokens()->default_value(false), "List tags")
+            ("tag", bpo::value<std::string>(&tags)->default_value(""), "Filter using tags. Separate with \",\" to use multiple values")
         ;
         // Commandline options (config file)
         options_cli_cfg.add_options()
@@ -243,11 +254,13 @@ int main(int argc, char *argv[])
             ("save-changelogs", bpo::value<bool>(&Globals::globalConfig.dlConf.bSaveChangelogs)->zero_tokens()->default_value(false), "Save changelogs when downloading")
             ("threads", bpo::value<unsigned int>(&Globals::globalConfig.iThreads)->default_value(4), "Number of download threads")
             ("info-threads", bpo::value<unsigned int>(&Globals::globalConfig.iInfoThreads)->default_value(4), "Number of threads for getting product info")
+            ("use-dlc-list", bpo::value<bool>(&bUseDLCList)->zero_tokens()->default_value(false), "Use DLC list specified with --dlc-list")
             ("dlc-list", bpo::value<std::string>(&Globals::globalConfig.sGameHasDLCList)->default_value("https://raw.githubusercontent.com/Sude-/lgogdownloader-lists/master/game_has_dlc.txt"), "Set URL for list of games that have DLC")
             ("progress-interval", bpo::value<int>(&Globals::globalConfig.iProgressInterval)->default_value(100), "Set interval for progress bar update (milliseconds)\nValue must be between 1 and 10000")
             ("lowspeed-timeout", bpo::value<long int>(&Globals::globalConfig.curlConf.iLowSpeedTimeout)->default_value(30), "Set time in number seconds that the transfer speed should be below the rate set with --lowspeed-rate for it to considered too slow and aborted")
             ("lowspeed-rate", bpo::value<long int>(&Globals::globalConfig.curlConf.iLowSpeedTimeoutRate)->default_value(200), "Set average transfer speed in bytes per second that the transfer should be below during time specified with --lowspeed-timeout for it to be considered too slow and aborted")
             ("include-hidden-products", bpo::value<bool>(&Globals::globalConfig.bIncludeHiddenProducts)->zero_tokens()->default_value(false), "Include games that have been set hidden in account page")
+            ("size-only", bpo::value<bool>(&Globals::globalConfig.bSizeOnly)->zero_tokens()->default_value(false), "Don't check the hashes of the files whose size matches that on the server")
         ;
 
         options_cli_no_cfg_hidden.add_options()
@@ -263,7 +276,7 @@ int main(int argc, char *argv[])
             ("galaxy-arch", bpo::value<std::string>(&sGalaxyArch)->default_value("x64"), galaxy_arch_text.c_str())
             ("galaxy-no-dependencies", bpo::value<bool>(&bNoGalaxyDependencies)->zero_tokens()->default_value(false), "Don't download dependencies during --galaxy-install")
             ("subdir-galaxy-install", bpo::value<std::string>(&Globals::globalConfig.dirConf.sGalaxyInstallSubdir)->default_value("%install_dir%"), galaxy_install_subdir_text.c_str())
-            ("galaxy-cdn-priority", bpo::value<std::string>(&sGalaxyCDN)->default_value("edgecast,highwinds,gog_cdn"), galaxy_cdn_priority_text.c_str())
+            ("galaxy-cdn-priority", bpo::value<std::string>(&sGalaxyCDN)->default_value("edgecast,highwinds,akamai,lumen,gog_cdn"), galaxy_cdn_priority_text.c_str())
             ("galaxy-delete-orphans", bpo::value<bool>(&Globals::globalConfig.dlConf.bGalaxyDeleteOrphans)->zero_tokens()->default_value(false), "Delete orphaned files during --galaxy-install")
         ;
 
@@ -379,9 +392,12 @@ int main(int argc, char *argv[])
             }
         }
 
+        if (!bUseDLCList)
+            Globals::globalConfig.sGameHasDLCList = "";
+
         if (Globals::globalConfig.sIgnoreDLCCountRegex.empty())
         {
-            if (boost::filesystem::exists(Globals::globalConfig.sGameHasDLCListFilePath))
+            if (boost::filesystem::exists(Globals::globalConfig.sGameHasDLCListFilePath) && bUseDLCList)
             {
                 std::ifstream ifs(Globals::globalConfig.sGameHasDLCListFilePath.c_str());
                 if (!ifs)
@@ -478,6 +494,9 @@ int main(int argc, char *argv[])
             vFileIdStrings = Util::tokenize(Globals::globalConfig.sFileIdString, ",");
         }
 
+        if (!tags.empty())
+            Globals::globalConfig.dlConf.vTags = Util::tokenize(tags, ",");
+
         if (!Globals::globalConfig.sOutputFilename.empty() && vFileIdStrings.size() > 1)
         {
             std::cerr << "Cannot specify an output file name when downloading multiple files." << std::endl;
@@ -589,7 +608,6 @@ int main(int argc, char *argv[])
     }
 
     // Init curl globally
-    ssl_thread_setup();
     curl_global_init(CURL_GLOBAL_ALL);
 
     Downloader downloader;
@@ -597,11 +615,13 @@ int main(int argc, char *argv[])
     int iLoginTries = 0;
     bool bLoginOK = false;
 
-    // Login because --login, --login-api or --login-website was used
+    // Login because --login was used
     if (Globals::globalConfig.bLogin)
         bLoginOK = downloader.login();
 
     bool bIsLoggedin = downloader.isLoggedIn();
+    if (!bIsLoggedin)
+        Globals::globalConfig.bLogin = true;
 
     // Login because we are not logged in
     while (iLoginTries++ < Globals::globalConfig.iRetries && !bIsLoggedin)
@@ -617,7 +637,6 @@ int main(int argc, char *argv[])
     if (!bLoginOK && !bIsLoggedin)
     {
         curl_global_cleanup();
-        ssl_thread_cleanup();
         return 1;
     }
 
@@ -681,7 +700,6 @@ int main(int argc, char *argv[])
             if (Globals::globalConfig.bSaveConfig)
             {
                 curl_global_cleanup();
-                ssl_thread_cleanup();
                 return 0;
             }
         }
@@ -689,7 +707,6 @@ int main(int argc, char *argv[])
         {
             std::cerr << "Failed to create config: " << Globals::globalConfig.sConfigFilePath << std::endl;
             curl_global_cleanup();
-            ssl_thread_cleanup();
             return 1;
         }
     }
@@ -703,14 +720,12 @@ int main(int argc, char *argv[])
                 Util::setFilePermissions(Globals::globalConfig.sConfigFilePath, boost::filesystem::owner_read | boost::filesystem::owner_write);
 
             curl_global_cleanup();
-            ssl_thread_cleanup();
             return 0;
         }
         else
         {
             std::cerr << "Failed to create config: " << Globals::globalConfig.sConfigFilePath << std::endl;
             curl_global_cleanup();
-            ssl_thread_cleanup();
             return 1;
         }
     }
@@ -719,7 +734,6 @@ int main(int argc, char *argv[])
     if (!bInitOK)
     {
         curl_global_cleanup();
-        ssl_thread_cleanup();
         return 1;
     }
 
@@ -746,6 +760,8 @@ int main(int argc, char *argv[])
         downloader.download();
     else if (Globals::globalConfig.bListDetails || Globals::globalConfig.bList) // Detailed list of games/extras
         res = downloader.listGames();
+    else if (Globals::globalConfig.bListTags) // List tags
+        res = downloader.listTags();
     else if (!Globals::globalConfig.sOrphanRegex.empty()) // Check for orphaned files if regex for orphans is set
         downloader.checkOrphans();
     else if (Globals::globalConfig.bCheckStatus)
@@ -787,7 +803,6 @@ int main(int argc, char *argv[])
         downloader.checkOrphans();
 
     curl_global_cleanup();
-    ssl_thread_cleanup();
 
     return res;
 }
