@@ -5,6 +5,7 @@
  * http://www.wtfpl.net/ for more details. */
 
 #include "galaxyapi.h"
+#include "ziputil.h"
 
 #include <boost/iostreams/filtering_streambuf.hpp>
 #include <boost/iostreams/copy.hpp>
@@ -26,25 +27,7 @@ galaxyAPI::galaxyAPI(CurlConfig& conf)
     this->curlConf = conf;
 
     curlhandle = curl_easy_init();
-    curl_easy_setopt(curlhandle, CURLOPT_FOLLOWLOCATION, 1);
-    curl_easy_setopt(curlhandle, CURLOPT_NOPROGRESS, 1);
-    curl_easy_setopt(curlhandle, CURLOPT_PROGRESSDATA, this);
-    curl_easy_setopt(curlhandle, CURLOPT_FAILONERROR, true);
-    curl_easy_setopt(curlhandle, CURLOPT_NOSIGNAL, 1);
-    curl_easy_setopt(curlhandle, CURLOPT_CONNECTTIMEOUT, curlConf.iTimeout);
-    curl_easy_setopt(curlhandle, CURLOPT_FAILONERROR, true);
-    curl_easy_setopt(curlhandle, CURLOPT_COOKIEFILE, curlConf.sCookiePath.c_str());
-    curl_easy_setopt(curlhandle, CURLOPT_COOKIEJAR, curlConf.sCookiePath.c_str());
-    curl_easy_setopt(curlhandle, CURLOPT_SSL_VERIFYPEER, curlConf.bVerifyPeer);
-    curl_easy_setopt(curlhandle, CURLOPT_VERBOSE, curlConf.bVerbose);
-    curl_easy_setopt(curlhandle, CURLOPT_MAX_RECV_SPEED_LARGE, curlConf.iDownloadRate);
-
-    // Assume that we have connection error and abort transfer with CURLE_OPERATION_TIMEDOUT if download speed is less than 200 B/s for 30 seconds
-    curl_easy_setopt(curlhandle, CURLOPT_LOW_SPEED_TIME, curlConf.iLowSpeedTimeout);
-    curl_easy_setopt(curlhandle, CURLOPT_LOW_SPEED_LIMIT, curlConf.iLowSpeedTimeoutRate);
-
-    if (!curlConf.sCACertPath.empty())
-        curl_easy_setopt(curlhandle, CURLOPT_CAINFO, curlConf.sCACertPath.c_str());
+    Util::CurlHandleSetDefaultOptions(curlhandle, this->curlConf);
 }
 
 galaxyAPI::~galaxyAPI()
@@ -97,10 +80,8 @@ bool galaxyAPI::isTokenExpired()
     return res;
 }
 
-std::string galaxyAPI::getResponse(const std::string& url, const bool& zlib_decompress)
+std::string galaxyAPI::getResponse(const std::string& url)
 {
-    std::ostringstream memory;
-
     struct curl_slist *header = NULL;
 
     std::string access_token;
@@ -112,44 +93,23 @@ std::string galaxyAPI::getResponse(const std::string& url, const bool& zlib_deco
         header = curl_slist_append(header, bearer.c_str());
     }
     curl_easy_setopt(curlhandle, CURLOPT_HTTPHEADER, header);
-
     curl_easy_setopt(curlhandle, CURLOPT_URL, url.c_str());
-    curl_easy_setopt(curlhandle, CURLOPT_NOPROGRESS, 1);
-    curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, galaxyAPI::writeMemoryCallback);
-    curl_easy_setopt(curlhandle, CURLOPT_WRITEDATA, &memory);
+    curl_easy_setopt(curlhandle, CURLOPT_ACCEPT_ENCODING, "");
 
-    int retries = 0;
-    CURLcode result;
+    int max_retries = std::min(3, Globals::globalConfig.iRetries);
     std::string response;
-    do
-    {
-        if (Globals::globalConfig.iWait > 0)
-            usleep(Globals::globalConfig.iWait); // Delay the request by specified time
-        result = curl_easy_perform(curlhandle);
-        response = memory.str();
-        memory.str(std::string());
-    }
-    while ((result != CURLE_OK) && response.empty() && (retries++ < Globals::globalConfig.iRetries));
+    Util::CurlHandleGetResponse(curlhandle, response, max_retries);
 
+    curl_easy_setopt(curlhandle, CURLOPT_ACCEPT_ENCODING, NULL);
     curl_easy_setopt(curlhandle, CURLOPT_HTTPHEADER, NULL);
     curl_slist_free_all(header);
 
-    if (zlib_decompress)
-    {
-        std::string response_decompressed;
-        boost::iostreams::filtering_streambuf<boost::iostreams::input> in;
-        in.push(boost::iostreams::zlib_decompressor(GlobalConstants::ZLIB_WINDOW_SIZE));
-        in.push(boost::make_iterator_range(response));
-        boost::iostreams::copy(in, boost::iostreams::back_inserter(response_decompressed));
-        response = response_decompressed;
-    }
-
     return response;
 }
 
-Json::Value galaxyAPI::getResponseJson(const std::string& url, const bool& zlib_decompress)
+Json::Value galaxyAPI::getResponseJson(const std::string& url)
 {
-    std::istringstream response(this->getResponse(url, zlib_decompress));
+    std::istringstream response(this->getResponse(url));
     Json::Value json;
 
     if (!response.str().empty())
@@ -161,6 +121,36 @@ Json::Value galaxyAPI::getResponseJson(const std::string& url, const bool& zlib_
         catch(const Json::Exception& exc)
         {
             // Failed to parse json response
+            // Check for zlib header and decompress if header found
+            response.seekg(0, response.beg);
+            uint16_t header = ZipUtil::readUInt16(&response);
+            std::vector<uint16_t> zlib_headers = { 0x0178, 0x5e78, 0x9c78, 0xda78 };
+            bool is_zlib_compressed = std::any_of(
+                                zlib_headers.begin(), zlib_headers.end(),
+                                [header](uint16_t zlib_header)
+                                {
+                                    return header == zlib_header;
+                                }
+                            );
+
+            if (is_zlib_compressed)
+            {
+                std::string response_compressed = response.str();
+                std::stringstream response_decompressed;
+                boost::iostreams::filtering_streambuf<boost::iostreams::input> in;
+                in.push(boost::iostreams::zlib_decompressor(GlobalConstants::ZLIB_WINDOW_SIZE));
+                in.push(boost::make_iterator_range(response_compressed));
+                boost::iostreams::copy(in, response_decompressed);
+
+                try
+                {
+                    response_decompressed >> json;
+                }
+                catch(const Json::Exception& exc)
+                {
+                    // Failed to parse json
+                }
+            }
         }
     }
 
@@ -197,7 +187,7 @@ Json::Value galaxyAPI::getManifestV2(std::string manifest_hash, const bool& is_d
     else
         url = "https://cdn.gog.com/content-system/v2/meta/" + manifest_hash;
 
-    return this->getResponseJson(url, true);
+    return this->getResponseJson(url);
 }
 
 Json::Value galaxyAPI::getSecureLink(const std::string& product_id, const std::string& path)
@@ -290,22 +280,22 @@ gameDetails galaxyAPI::productInfoJsonToGameDetails(const Json::Value& json, con
 
     if (dlConf.bInstallers)
     {
-        gamedetails.installers = this->installerJsonNodeToGameFileVector(gamedetails.gamename, json["downloads"]["installers"], dlConf);
+        gamedetails.installers = this->fileJsonNodeToGameFileVector(gamedetails.gamename, json["downloads"]["installers"], GFTYPE_INSTALLER, dlConf);
     }
 
     if (dlConf.bExtras)
     {
-        gamedetails.extras = this->extraJsonNodeToGameFileVector(gamedetails.gamename, json["downloads"]["bonus_content"]);
+        gamedetails.extras = this->fileJsonNodeToGameFileVector(gamedetails.gamename, json["downloads"]["bonus_content"], GFTYPE_EXTRA, dlConf);
     }
 
     if (dlConf.bPatches)
     {
-        gamedetails.patches = this->patchJsonNodeToGameFileVector(gamedetails.gamename, json["downloads"]["patches"], dlConf);
+        gamedetails.patches = this->fileJsonNodeToGameFileVector(gamedetails.gamename, json["downloads"]["patches"], GFTYPE_PATCH, dlConf);
     }
 
     if (dlConf.bLanguagePacks)
     {
-        gamedetails.languagepacks = this->languagepackJsonNodeToGameFileVector(gamedetails.gamename, json["downloads"]["language_packs"], dlConf);
+        gamedetails.languagepacks = this->fileJsonNodeToGameFileVector(gamedetails.gamename, json["downloads"]["language_packs"], GFTYPE_LANGPACK, dlConf);
     }
 
     if (dlConf.bDLC)
@@ -336,27 +326,7 @@ gameDetails galaxyAPI::productInfoJsonToGameDetails(const Json::Value& json, con
     return gamedetails;
 }
 
-std::vector<gameFile> galaxyAPI::installerJsonNodeToGameFileVector(const std::string& gamename, const Json::Value& json, const DownloadConfig& dlConf)
-{
-    return this->fileJsonNodeToGameFileVector(gamename, json, GFTYPE_INSTALLER, dlConf.iInstallerPlatform, dlConf.iInstallerLanguage, dlConf.bDuplicateHandler);
-}
-
-std::vector<gameFile> galaxyAPI::patchJsonNodeToGameFileVector(const std::string& gamename, const Json::Value& json, const DownloadConfig& dlConf)
-{
-    return this->fileJsonNodeToGameFileVector(gamename, json, GFTYPE_PATCH, dlConf.iInstallerPlatform, dlConf.iInstallerLanguage, dlConf.bDuplicateHandler);
-}
-
-std::vector<gameFile> galaxyAPI::languagepackJsonNodeToGameFileVector(const std::string& gamename, const Json::Value& json, const DownloadConfig& dlConf)
-{
-    return this->fileJsonNodeToGameFileVector(gamename, json, GFTYPE_LANGPACK, dlConf.iInstallerPlatform, dlConf.iInstallerLanguage, dlConf.bDuplicateHandler);
-}
-
-std::vector<gameFile> galaxyAPI::extraJsonNodeToGameFileVector(const std::string& gamename, const Json::Value& json)
-{
-    return this->fileJsonNodeToGameFileVector(gamename, json, GFTYPE_EXTRA);
-}
-
-std::vector<gameFile> galaxyAPI::fileJsonNodeToGameFileVector(const std::string& gamename, const Json::Value& json, const unsigned int& type, const unsigned int& platform, const unsigned int& lang, const bool& useDuplicateHandler)
+std::vector<gameFile> galaxyAPI::fileJsonNodeToGameFileVector(const std::string& gamename, const Json::Value& json, const unsigned int& type, const DownloadConfig& dlConf)
 {
     std::vector<gameFile> gamefiles;
     unsigned int iInfoNodes = json.size();
@@ -365,6 +335,9 @@ std::vector<gameFile> galaxyAPI::fileJsonNodeToGameFileVector(const std::string&
         Json::Value infoNode = json[i];
         unsigned int iFiles = infoNode["files"].size();
         std::string name = infoNode["name"].asString();
+        std::string version = "";
+        if (!infoNode["version"].empty())
+            version = infoNode["version"].asString();
 
         unsigned int iPlatform = GlobalConstants::PLATFORM_WINDOWS;
         unsigned int iLanguage = GlobalConstants::LANGUAGE_EN;
@@ -373,13 +346,20 @@ std::vector<gameFile> galaxyAPI::fileJsonNodeToGameFileVector(const std::string&
             iPlatform = Util::getOptionValue(infoNode["os"].asString(), GlobalConstants::PLATFORMS);
             iLanguage = Util::getOptionValue(infoNode["language"].asString(), GlobalConstants::LANGUAGES);
 
-            if (!(iPlatform & platform))
+            if (!(iPlatform & dlConf.iInstallerPlatform))
                 continue;
 
-            if (!(iLanguage & lang))
+            if (!(iLanguage & dlConf.iInstallerLanguage))
                 continue;
         }
 
+        // Skip file if count and total_size is zero
+        // https://github.com/Sude-/lgogdownloader/issues/200
+        unsigned int count = infoNode["count"].asUInt();
+        uintmax_t total_size = infoNode["total_size"].asLargestUInt();
+        if (count == 0 && total_size == 0)
+            continue;
+
         for (unsigned int j = 0; j < iFiles; ++j)
         {
             Json::Value fileNode = infoNode["files"][j];
@@ -392,6 +372,12 @@ std::vector<gameFile> galaxyAPI::fileJsonNodeToGameFileVector(const std::string&
             std::string downlink_url = downlinkJson["downlink"].asString();
             std::string path = this->getPathFromDownlinkUrl(downlink_url, gamename);
 
+            // Check to see if path ends in "/secure" which means that we got invalid path for some reason
+            boost::regex path_re("/secure$", boost::regex::perl | boost::regex::icase);
+            boost::match_results<std::string::const_iterator> what;
+            if (boost::regex_search(path, what, path_re))
+                continue;
+
             gameFile gf;
             gf.gamename = gamename;
             gf.type = type;
@@ -401,27 +387,29 @@ std::vector<gameFile> galaxyAPI::fileJsonNodeToGameFileVector(const std::string&
             gf.size = Util::getJsonUIntValueAsString(fileNode["size"]);
             gf.updated = 0; // assume not updated
             gf.galaxy_downlink_json_url = downlink;
+            gf.version = version;
 
             if (!(type & GFTYPE_EXTRA))
             {
                 gf.platform = iPlatform;
                 gf.language = iLanguage;
+            }
 
-                if (useDuplicateHandler)
+            if (dlConf.bDuplicateHandler)
+            {
+                bool bDuplicate = false;
+                for (unsigned int k = 0; k < gamefiles.size(); ++k)
                 {
-                    bool bDuplicate = false;
-                    for (unsigned int k = 0; k < gamefiles.size(); ++k)
+                    if (gamefiles[k].path == gf.path)
                     {
-                        if (gamefiles[k].path == gf.path)
-                        {
+                        if (!(type & GFTYPE_EXTRA))
                             gamefiles[k].language |= gf.language; // Add language code to installer
-                            bDuplicate = true;
-                            break;
-                        }
+                        bDuplicate = true;
+                        break;
                     }
-                    if (bDuplicate)
-                        continue;
                 }
+                if (bDuplicate)
+                    continue;
             }
 
             gamefiles.push_back(gf);
@@ -449,7 +437,7 @@ Json::Value galaxyAPI::getDependenciesJson()
         if (repository.isMember("repository_manifest"))
         {
             std::string manifest_url = repository["repository_manifest"].asString();
-            dependencies = this->getResponseJson(manifest_url, true);
+            dependencies = this->getResponseJson(manifest_url);
         }
     }
 
@@ -552,3 +540,93 @@ std::string galaxyAPI::getPathFromDownlinkUrl(const std::string& downlink_url, c
 
     return path;
 }
+
+std::vector<std::string> galaxyAPI::cdnUrlTemplatesFromJson(const Json::Value& json, const std::vector<unsigned int>& cdnPriority)
+{
+    // Handle priority of CDNs
+    struct urlPriority
+    {
+        std::string url;
+        int priority;
+    };
+    std::vector<urlPriority> cdnUrls;
+
+    // Build a vector of all urls and their priority score
+    for (unsigned int i = 0; i < json["urls"].size(); ++i)
+    {
+        std::string endpoint_name = json["urls"][i]["endpoint_name"].asString();
+
+        unsigned int score = cdnPriority.size();
+        unsigned int cdn = Util::getOptionValue(endpoint_name, GlobalConstants::GALAXY_CDNS, false);
+        for (unsigned int idx = 0; idx < score; ++idx)
+        {
+            if (cdn & cdnPriority[idx])
+            {
+                score = idx;
+                break;
+            }
+        }
+
+        // Couldn't find a match when assigning score
+        if (score == cdnPriority.size())
+        {
+            // Add index value to score
+            // This way unknown CDNs have priority based on the order they appear in json
+            score += i;
+        }
+
+        // Build url according to url_format
+        std::string link_base_url = json["urls"][i]["parameters"]["base_url"].asString();
+        std::string link_path = json["urls"][i]["parameters"]["path"].asString();
+        std::string link_token = json["urls"][i]["parameters"]["token"].asString();
+
+        // Add our own template to path
+        link_path += "{LGOGDOWNLOADER_GALAXY_PATH}";
+
+        std::string url = json["urls"][i]["url_format"].asString();
+
+        while(Util::replaceString(url, "{base_url}", link_base_url));
+        while(Util::replaceString(url, "{path}", link_path));
+        while(Util::replaceString(url, "{token}", link_token));
+
+        // Highwinds specific
+        std::string link_hw_l= json["urls"][i]["parameters"]["l"].asString();
+        std::string link_hw_source = json["urls"][i]["parameters"]["source"].asString();
+        std::string link_hw_ttl = json["urls"][i]["parameters"]["ttl"].asString();
+        std::string link_hw_gog_token = json["urls"][i]["parameters"]["gog_token"].asString();
+
+        while(Util::replaceString(url, "{l}", link_hw_l));
+        while(Util::replaceString(url, "{source}", link_hw_source));
+        while(Util::replaceString(url, "{ttl}", link_hw_ttl));
+        while(Util::replaceString(url, "{gog_token}", link_hw_gog_token));
+
+        // Lumen specific
+        std::string dirs = json["urls"][i]["parameters"]["dirs"].asString();
+        std::string expires_at = json["urls"][i]["parameters"]["expires_at"].asString();
+
+        while(Util::replaceString(url, "{dirs}", dirs));
+        while(Util::replaceString(url, "{expires_at}", expires_at));
+
+        urlPriority cdnurl;
+        cdnurl.url = url;
+        cdnurl.priority = score;
+        cdnUrls.push_back(cdnurl);
+    }
+
+    if (!cdnUrls.empty())
+    {
+        // Sort urls by priority (lowest score first)
+        std::sort(cdnUrls.begin(), cdnUrls.end(),
+            [](urlPriority a, urlPriority b)
+            {
+                return (a.priority < b.priority);
+            }
+        );
+    }
+
+    std::vector<std::string> cdnUrlTemplates;
+    for (auto cdnurl : cdnUrls)
+        cdnUrlTemplates.push_back(cdnurl.url);
+
+    return cdnUrlTemplates;
+}
