@@ -19,24 +19,7 @@ Website::Website()
     this->retries = 0;
 
     curlhandle = curl_easy_init();
-    curl_easy_setopt(curlhandle, CURLOPT_FOLLOWLOCATION, 1);
-    curl_easy_setopt(curlhandle, CURLOPT_USERAGENT, Globals::globalConfig.curlConf.sUserAgent.c_str());
-    curl_easy_setopt(curlhandle, CURLOPT_NOPROGRESS, 1);
-    curl_easy_setopt(curlhandle, CURLOPT_NOSIGNAL, 1);
-    curl_easy_setopt(curlhandle, CURLOPT_CONNECTTIMEOUT, Globals::globalConfig.curlConf.iTimeout);
-    curl_easy_setopt(curlhandle, CURLOPT_FAILONERROR, true);
-    curl_easy_setopt(curlhandle, CURLOPT_COOKIEFILE, Globals::globalConfig.curlConf.sCookiePath.c_str());
-    curl_easy_setopt(curlhandle, CURLOPT_COOKIEJAR, Globals::globalConfig.curlConf.sCookiePath.c_str());
-    curl_easy_setopt(curlhandle, CURLOPT_SSL_VERIFYPEER, Globals::globalConfig.curlConf.bVerifyPeer);
-    curl_easy_setopt(curlhandle, CURLOPT_VERBOSE, Globals::globalConfig.curlConf.bVerbose);
-    curl_easy_setopt(curlhandle, CURLOPT_MAX_RECV_SPEED_LARGE, Globals::globalConfig.curlConf.iDownloadRate);
-
-    // Assume that we have connection error and abort transfer with CURLE_OPERATION_TIMEDOUT if download speed is less than 200 B/s for 30 seconds
-    curl_easy_setopt(curlhandle, CURLOPT_LOW_SPEED_TIME, 30);
-    curl_easy_setopt(curlhandle, CURLOPT_LOW_SPEED_LIMIT, 200);
-
-    if (!Globals::globalConfig.curlConf.sCACertPath.empty())
-        curl_easy_setopt(curlhandle, CURLOPT_CAINFO, Globals::globalConfig.curlConf.sCACertPath.c_str());
+    Util::CurlHandleSetDefaultOptions(curlhandle, Globals::globalConfig.curlConf);
 }
 
 Website::~Website()
@@ -44,34 +27,14 @@ Website::~Website()
     curl_easy_cleanup(curlhandle);
 }
 
-size_t Website::writeMemoryCallback(char *ptr, size_t size, size_t nmemb, void *userp)
-{
-    std::ostringstream *stream = (std::ostringstream*)userp;
-    size_t count = size * nmemb;
-    stream->write(ptr, count);
-    return count;
-}
-
 std::string Website::getResponse(const std::string& url)
 {
-    std::ostringstream memory;
     std::string response;
 
     curl_easy_setopt(curlhandle, CURLOPT_URL, url.c_str());
-    curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, Website::writeMemoryCallback);
-    curl_easy_setopt(curlhandle, CURLOPT_WRITEDATA, &memory);
+    int max_retries = std::min(3, Globals::globalConfig.iRetries);
 
-    CURLcode result;
-    do
-    {
-        if (Globals::globalConfig.iWait > 0)
-            usleep(Globals::globalConfig.iWait); // Delay the request by specified time
-        result = curl_easy_perform(curlhandle);
-        response = memory.str();
-        memory.str(std::string());
-    }
-    while ((result != CURLE_OK) && response.empty() && (this->retries++ < Globals::globalConfig.iRetries));
-    this->retries = 0; // reset retries counter
+    CURLcode result = Util::CurlHandleGetResponse(curlhandle, response, max_retries);
 
     if (result != CURLE_OK)
     {
@@ -129,10 +92,21 @@ std::vector<gameItem> Website::getGames()
     bool bAllPagesParsed = false;
     int iUpdated = Globals::globalConfig.bUpdated ? 1 : 0;
     int iHidden = 0;
+    std::string tags;
+    for (auto tag : Globals::globalConfig.dlConf.vTags)
+    {
+        if (tags.empty())
+            tags = tag;
+        else
+            tags += "," + tag;
+    }
 
     do
     {
-        std::string response = this->getResponse("https://www.gog.com/account/getFilteredProducts?hiddenFlag=" + std::to_string(iHidden) + "&isUpdated=" + std::to_string(iUpdated) + "&mediaType=1&sortBy=title&system=&page=" + std::to_string(i));
+        std::string url = "https://www.gog.com/account/getFilteredProducts?hiddenFlag=" + std::to_string(iHidden) + "&isUpdated=" + std::to_string(iUpdated) + "&mediaType=1&sortBy=title&system=&page=" + std::to_string(i);
+        if (!tags.empty())
+            url += "&tags=" + tags;
+        std::string response = this->getResponse(url);
         std::istringstream json_stream(response);
 
         try {
@@ -307,7 +281,7 @@ int Website::Login(const std::string& email, const std::string& password)
         std::cerr << "DEBUG INFO (Website::Login)" << std::endl;
         std::cerr << login_form_html << std::endl;
     #endif
-    if (login_form_html.find("google.com/recaptcha") != std::string::npos)
+    if (login_form_html.find("class=\"g-recaptcha form__recaptcha\"") != std::string::npos)
     {
         bRecaptcha = true;
         #ifndef USE_QT_GUI_LOGIN
@@ -373,7 +347,7 @@ int Website::Login(const std::string& email, const std::string& password)
         curl_easy_setopt(curlhandle, CURLOPT_URL, "https://login.gog.com/login_check");
         curl_easy_setopt(curlhandle, CURLOPT_POST, 1);
         curl_easy_setopt(curlhandle, CURLOPT_POSTFIELDS, postdata.c_str());
-        curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, Website::writeMemoryCallback);
+        curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, Util::CurlWriteMemoryCallback);
         curl_easy_setopt(curlhandle, CURLOPT_WRITEDATA, &memory);
         curl_easy_setopt(curlhandle, CURLOPT_NOPROGRESS, 1);
         curl_easy_setopt(curlhandle, CURLOPT_MAXREDIRS, 0);
@@ -391,12 +365,9 @@ int Website::Login(const std::string& email, const std::string& password)
                 std::cout << curl_easy_strerror(result) << std::endl;
         }
 
-        // Get redirect url
-        char *redirect_url;
-        curl_easy_getinfo(curlhandle, CURLINFO_REDIRECT_URL, &redirect_url);
-
         // Handle two step authorization
-        if (std::string(redirect_url).find("two_step") != std::string::npos)
+        std::string redirect_url = Util::CurlHandleGetInfoString(curlhandle, CURLINFO_REDIRECT_URL);
+        if (redirect_url.find("two_step") != std::string::npos)
         {
             std::string security_code;
             std::string tagname_two_step_send = "second_step_authentication[send]";
@@ -407,7 +378,7 @@ int Website::Login(const std::string& email, const std::string& password)
             std::string tagname_two_step_token;
             std::string token_two_step;
             std::string two_step_html = this->getResponse(redirect_url);
-            redirect_url = NULL;
+            redirect_url = "";
 
             tree<htmlcxx::HTML::Node> two_step_dom = parser.parseTree(two_step_html);
             tree<htmlcxx::HTML::Node>::iterator two_step_it = two_step_dom.begin();
@@ -443,7 +414,7 @@ int Website::Login(const std::string& email, const std::string& password)
             curl_easy_setopt(curlhandle, CURLOPT_URL, "https://login.gog.com/login/two_step");
             curl_easy_setopt(curlhandle, CURLOPT_POST, 1);
             curl_easy_setopt(curlhandle, CURLOPT_POSTFIELDS, postdata.c_str());
-            curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, Website::writeMemoryCallback);
+            curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, Util::CurlWriteMemoryCallback);
             curl_easy_setopt(curlhandle, CURLOPT_WRITEDATA, &memory);
             curl_easy_setopt(curlhandle, CURLOPT_NOPROGRESS, 1);
             curl_easy_setopt(curlhandle, CURLOPT_MAXREDIRS, 0);
@@ -453,26 +424,25 @@ int Website::Login(const std::string& email, const std::string& password)
             curl_easy_setopt(curlhandle, CURLOPT_FOLLOWLOCATION, 0);
             result = curl_easy_perform(curlhandle);
             memory.str(std::string());
-            curl_easy_getinfo(curlhandle, CURLINFO_REDIRECT_URL, &redirect_url);
+            redirect_url = Util::CurlHandleGetInfoString(curlhandle, CURLINFO_REDIRECT_URL);
         }
 
-        if (!std::string(redirect_url).empty())
+        if (!redirect_url.empty())
         {
             long response_code;
             do
             {
-                curl_easy_setopt(curlhandle, CURLOPT_URL, redirect_url);
+                curl_easy_setopt(curlhandle, CURLOPT_URL, redirect_url.c_str());
                 result = curl_easy_perform(curlhandle);
                 memory.str(std::string());
 
                 result = curl_easy_getinfo(curlhandle, CURLINFO_RESPONSE_CODE, &response_code);
                 if ((response_code / 100) == 3)
-                    curl_easy_getinfo(curlhandle, CURLINFO_REDIRECT_URL, &redirect_url);
+                    redirect_url = Util::CurlHandleGetInfoString(curlhandle, CURLINFO_REDIRECT_URL);
 
-                std::string redir_url = std::string(redirect_url);
                 boost::regex re(".*code=(.*?)([\?&].*|$)", boost::regex_constants::icase);
                 boost::match_results<std::string::const_iterator> what;
-                if (boost::regex_search(redir_url, what, re))
+                if (boost::regex_search(redirect_url, what, re))
                 {
                     auth_code = what[1];
                     if (!auth_code.empty())
@@ -481,7 +451,7 @@ int Website::Login(const std::string& email, const std::string& password)
             } while (result == CURLE_OK && (response_code / 100) == 3);
         }
 
-        curl_easy_setopt(curlhandle, CURLOPT_URL, redirect_url);
+        curl_easy_setopt(curlhandle, CURLOPT_URL, redirect_url.c_str());
         curl_easy_setopt(curlhandle, CURLOPT_HTTPGET, 1);
         curl_easy_setopt(curlhandle, CURLOPT_MAXREDIRS, -1);
         curl_easy_setopt(curlhandle, CURLOPT_FOLLOWLOCATION, 1);
@@ -597,7 +567,7 @@ bool Website::IsloggedInSimple()
 
     curl_easy_setopt(curlhandle, CURLOPT_URL, url.c_str());
     curl_easy_setopt(curlhandle, CURLOPT_FOLLOWLOCATION, 0);
-    curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, Website::writeMemoryCallback);
+    curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, Util::CurlWriteMemoryCallback);
     curl_easy_setopt(curlhandle, CURLOPT_WRITEDATA, &memory);
     curl_easy_setopt(curlhandle, CURLOPT_NOPROGRESS, 1);
     curl_easy_perform(curlhandle);
@@ -721,3 +691,50 @@ std::vector<wishlistItem> Website::getWishlistItems()
 
     return wishlistItems;
 }
+
+std::map<std::string, std::string> Website::getTags()
+{
+    std::string url = "https://www.gog.com/account/getFilteredProducts?mediaType=1&sortBy=title&system=&page=1";
+    std::string response = this->getResponse(url);
+    std::istringstream json_stream(response);
+    Json::Value json;
+    std::map<std::string, std::string> tags;
+
+    try
+    {
+        // Parse JSON
+        json_stream >> json;
+    }
+    catch (const Json::Exception& exc)
+    {
+        std::cout << exc.what();
+        if (!response.empty())
+        {
+            if(response[0] != '{')
+            {
+                // Response was not JSON. Assume that cookies have expired.
+                std::cerr << "Response was not JSON. Cookies have most likely expired. Try --login first." << std::endl;
+            }
+        }
+        exit(1);
+    }
+
+    tags = this->getTagsFromJson(json["tags"]);
+
+    return tags;
+}
+
+std::map<std::string, std::string> Website::getTagsFromJson(const Json::Value& json)
+{
+    std::map<std::string, std::string> tags;
+
+    if (!json.empty())
+    {
+        for (auto node : json)
+        {
+            tags[node["id"].asString()] = node["name"].asString();
+        }
+    }
+
+    return tags;
+}
